/***
   NAME
     chinook1.c
***/

/*==========================================================================
 *			INCLUDING THE HEADER FILE
 *==========================================================================
 */
#include  "escbox.h"

/*
 *==========================================================================
 *		LABELLING ENVIRONMENT AND I-STATE VARIABLES
 *==========================================================================
 */

#define time 		env[0]
#define	age		i_state(0)
#define	deathrate	i_const(0)


/*
 *==========================================================================
 *		DEFINING AND LABELLING CONSTANTS AND PARAMETERS
 *==========================================================================
 */
#define MU0		parameter[0]
#define MU1		parameter[1]
#define MUJ		parameter[2]
#define E               parameter[3]
#define P1              parameter[4]
#define P2              parameter[5]
#define P3              parameter[6]
#define P4              parameter[7]

#define YEAR            100.0
#define SMALL           0.01


/*
 *==========================================================================
 * USER INITIALIZATION ROUTINE ALLOWS OPERATIONS ON INITIAL POPULATIONS
 *==========================================================================
 */

void	UserInit( int argc, char **argv, double *env,  population *pop)
{

  fprintf(stderr, "P1 = %f\n", P1);
  fprintf(stderr, "P2 = %f\n", P2);
  fprintf(stderr, "P3 = %f\n", P3);
  fprintf(stderr, "P4 = %f\n", P4);
  return;
}

/*
 *==========================================================================
 *	SPECIFICATION OF THE NUMBER AND VALUES OF BOUNDARY POINTS
 *==========================================================================
 */

void	SetBpointNo(double *env, population *pop, int *bpoint_no)
{
  bpoint_no[0]=1;
  
  return;
}

/*==========================================================================*/

void	SetBpoints(double *env, population *pop, population *bpoints)
{
  bpoints[0][0][age]  = 0.0; 
  
  return;
}


/*
 *==========================================================================
 *			SPECIFICATION OF DERIVATIVES
 *==========================================================================
 */

void	Gradient(double *env,     population *pop,     population *ofs,
		 double *envgrad, population *popgrad, population *ofsgrad,
		 population *bpoints)
{
  register int		i;
  int                   a;

  for(i=0; i<cohort_no[0]; i++)	 
    {
      
      if (pop[0][i][age] < 1.0*YEAR)
	popIDcard[0][i][deathrate]  = MU0 + MU1*pop[0][i][number];
      else if (pop[0][i][age] < 2.0*YEAR)
	popIDcard[0][i][deathrate]  = MUJ;
      else 
	popIDcard[0][i][deathrate]  = 0.0;
    }
  
  for(i=0; i<cohort_no[0]; i++)			
    {						
      popgrad[0][i][number] = -popIDcard[0][i][deathrate]*pop[0][i][number];
      popgrad[0][i][age]    = 1.0;
    }
  
  envgrad[0] = 1.0;
  
  return;
}



/*
 *==========================================================================
 *		SPECIFICATION OF BETWEEN COHORT CYCLE DYNAMICS
 *==========================================================================
 */

void	InstantDynamics(double *env, population *pop, population *ofs)
{
  double                date, pr, juveniles;
  register int		i;
  double                a;


  /* What is the date? */
  date = fmod(time, YEAR);
  if (date != 0.0)
    return;                
  /* Return if within season, otherwise opportunity to reproduce */
 
  /* determine age of cohorts at time T- */
  for (i=0, juveniles=0.0; i<cohort_no[0]; i++)
    {
      a = floor((pop[0][i][age]+SMALL)/YEAR);
      pr = 0.0;
      if (a==1)
	pr = P1;
      else if (a==2)
	pr = P2;
      else if (a==3)
	pr = P3;
      else if (a==4)
	pr = P4;
      
      juveniles += pop[0][i][number]*E*pr;
      pop[0][i][number] = (1.0-pr)*pop[0][i][number];
      
      fprintf(stderr, "(reprod) t=%f, age of cohort %d is %d, p=%f\n", time,i,a,pr);
    }  
  
  ofs[0][0][number]             = juveniles;
  ofs[0][0][age]                = 0.0;
  ofsIDcard[0][0][deathrate]    = MU0 + MU1*ofs[0][0][number];
  
  return;
}



/*
 *==========================================================================
 *			SPECIFICATION OF OUTPUT VARIABLES
 *==========================================================================
 */

void	DefineOutput(double *env, population *pop, double *output)
{

  double                totpop, age0, age1, age2, age3, age4;
  register int		i;	
  int                   a;
  
  for(i=0, totpop=0.0; i<cohort_no[0]; i++)
    {
      totpop += pop[0][i][number];
    }	
  
  /* count individuals in three age classes */
  age0=0.0;
  age1=0.0;
  age2=0.0;
  age3=0.0;
  age4=0.0;
  for(i=0; i<cohort_no[0]; i++)		      
    {
      a = floor((pop[0][i][age]+SMALL)/YEAR);
      if (a==0)
	age0 = pop[0][i][number];
      else if (a==1)
	age1 = pop[0][i][number];
      else if (a==2)
	age2 = pop[0][i][number];
      else if (a==3)
	age3 = pop[0][i][number];
      else if (a==4)
	age4 = pop[0][i][number];
    }

  output[0] = totpop;
  output[1] = age0;
  output[2] = age1;
  output[3] = age2;
  output[4] = age3;
  output[5] = cohort_no[0];
  
  return;
}
/*==========================================================================*/
